/*
*
* pmc module.  Support for the PPro and PII performance monitoring
* counters.
*
* thain@primenet.com
*/
#include <linux/config.h>

#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h> /* for -EBUSY */

#include "pmc.h"

/* Performance Event select registers */
#define PERFEVENTSEL0 390
#define PERFEVENTSEL1 391

/* Performance counterregisters */
#define PERFCTR0 193
#define PERFCTR1 194

static int pmc_major; /* device major number */

static int pmc_eventsel0 = 0; /* what counter 0 is counting */
static int pmc_eventsel1 = 0; /* what counter 1 is counting */

static int open_pmc(struct inode *i, struct file *f) {
  MOD_INC_USE_COUNT;
  return 0;
}

static void release_pmc(struct inode *i, struct file *f) {
  MOD_DEC_USE_COUNT;
}

/* To clear the register, we must counters must be disabled! */
static void pmc_clear(int counter) {
    int counterRegister = PERFCTR0 + counter;

    /* clear the old register */
    __asm__("mov %0, %%ecx\n\t"
	  "xor %%edx, %%edx\n\t"
	  "xor %%eax, %%eax\n\t"
	  "wrmsr\n\t"
	  : /* no outputs */
	  : "m" (counterRegister)
	  : "eax", "ecx", "edx" /* all clobbered */);
}

static void setCounter(int counter, unsigned long arg) {
  if ((counter < 0) || (counter > 1)) {
    printk("pmc:setCounter illegal value for counter\n");
  } else {
    int selectionRegister = PERFEVENTSEL0 + counter;
    int counterRegister   = PERFCTR0 + counter;

    pmc_clear(counter);

    /* set the value */
    __asm__("mov %0, %%ecx\n\t"   /* ecx contains the number of the MSR to set */
	    "xor %%edx, %%edx\n\t"/* edx contains the high bits to set the MSR to */
	    "mov %1, %%eax\n\t"    /* eax contains the log bits to set the MSR to */
	    "wrmsr\n\t"
	    : /* no outputs */
	    : "m" (selectionRegister), "m" (arg)
	    : "eax", "ecx", "edx" /* clobbered */);
  }
}

static int ioctl_pmc(struct inode *inode, struct file *f,
			      unsigned int cmd, unsigned long arg) {
  switch (cmd) {
  case 0:
    /* disable counter */
    setCounter(0, 0);

    pmc_clear(0);

    /* set counter 0 */
    setCounter(0, arg);
    break;
  case 1:
    /* disable counter */
    setCounter(1, 0);

    pmc_clear(1);

    /* set counter 1 */
    setCounter(1, arg);
    break;
  default:
    printk("ioctl_pmc: illegal cmd: %d\n", cmd);
    break;
  }
  return 0;
}

static struct file_operations pmc_fops = {
  NULL,         /* no special lseek */
  NULL,         /* no special read  */
  NULL,         /* no special write */
  NULL,         /* no special readdir */
  NULL,         /* no special select */
  ioctl_pmc,      
  NULL,         /* no special mmap */
  open_pmc,
  release_pmc,
  NULL,         /* no special fsync */
  NULL,         /* no special fasync */
  NULL,         /* no special check_media_change */
  NULL,         /* no special revalidate */
};

/* only used when not compiled as a module */
long pmc_init(long mem_start, long mem_end) {
  init_module();
  return mem_start;
}

int init_module(void)
{
  int a;
  if ((pmc_major = register_chrdev(0, "pmc", &pmc_fops)) 
      == -EBUSY) {
    printk("unable to get major for pmc device\n");
    return -EIO;
  }

  /* 
     In order to use the rdpmc instruction in user mode, we need to set the
     PCE bit of CR4. PCE is 8th bit of cr4, and 256 is 2 << 8
  */
  __asm__("movl %%cr4, %0\n\t" /* move CR4 into a */
	  "orl $256, %0\n\t"  /* turn on PCE bit in a */
	  "movl %0, %%cr4\n" /* copy a back into CR4  */
	  : "=r" (a) /* writes to registerized a */
	  : /* no inputs */);

  return 0;
}

void cleanup_module(void)
{
  unregister_chrdev(pmc_major, "pmc");
}
