#include "pmc.h"
#include "rdpmc.h"

#include <fcntl.h>

typedef struct _pmcEvent {
  int type;
  char *shortDescription;
  char *longDescription;
} pmcEvent;

pmcEvent theEvents[] = {
DATA_MEM_REFS, "DATA_MEM_REFS", "All memory references, both cacheable and noncacheable.",
DCU_LINES_IN, "DCU_LINES_IN", "Total lines allocated in the DCU.",
DCU_M_LINES_IN, "DCU_M_LINES_IN", "Number of M state lines allocated in the DCU.",
DCU_M_LINES_OUT, "DCU_M_LINES_OUT", "Number of M state lines evicted from the DCU.",
DCU_MISS_OUTSTANDING, "DCU_MISS_OUTSTANDING", "Weighted number of cycles while a DCU miss is outstanding.",
IFU_IFETCH, "IFU_IFETCH", "Number of instruction fetches, both cacheable and noncacheable.",
IFU_IFETCH_MISS, "IFU_IFETCH_MISS", "Number of instruction fetch misses.",
ITLB_MISS, "ITLB_MISS", "Number of ITLB misses.",
IFU_MEM_STALL, "IFU_MEM_STALL", "Number of cycles that the instruction fetch pipe stage is stalled, including cache misses, ITLB misses, ITLB faults, and victimcache evictions.",
ILD_STALL, "ILD_STALL", "Number of cycles that the instruction length decoder is stalled.",
L2_IFETCH, "L2_IFETCH", "Number of L2 instruction fetches.",
L2_LD, "L2_LD", "Number of L2 data loads.",
L2_ST, "L2_ST", "Number of L2 data stores.",
L2_LINES_IN, "L2_LINES_IN", "Number of lines allocated in the L2.",
L2_LINES_OUT, "L2_LINES_OUT", "Number of lines removed from the L2 for any reason.",
L2_M_LINES_INM, "L2_M_LINES_INM", "Number of modified lines allocated in the L2.",
L2_M_LINES_OUTM, "L2_M_LINES_OUTM", "Number of modified lines removed from the L2 for any reason.",
L2_RQSTS, "L2_RQSTS", "Number of L2 requests.",
L2_ADS, "L2_ADS", "Number of L2 address strobes.",
L2_DBUS_BUSY, "L2_DBUS_BUSY", "Number of cycles during which the data bus was busy.",
L2_DBUS_BUSY_RD, "L2_DBUS_BUSY_RD", "Number of cycles during which the data bus was busy transferring data from L2 to the processor.",
BUS_DRDY_CLOCKS_SELF, "BUS_DRDY_CLOCKS_SELF", "Number of clocks during which DRDY is asserted by CPU.",
BUS_DRDY_CLOCKS_ANY, "BUS_DRDY_CLOCKS_ANY", "(Any) Number of clocks during which DRDY is asserted by any agent.",
BUS_LOCK_CLOCKS_SELF, "BUS_LOCK_CLOCKS_SELF", "Number of clocks during which LOCK is asserted.",
BUS_LOCK_CLOCKS_ANY, "BUS_LOCK_CLOCKS_ANY", "Number of clocks during which LOCK is asserted.",
BUS_REQ_OUTSTANDING_SELF, "BUS_REQ_OUTSTANDING_SELF", "Number of bus requests outstanding.",
BUS_REQ_OUTSTANDING_ANY, "BUS_REQ_OUTSTANDING_ANY", "Number of bus requests outstanding.",
BUS_TRAN_BRD_SELF, "BUS_TRAN_BRD_SELF", "Number of burst read transactions.",
BUS_TRAN_BRD_ANY, "BUS_TRAN_BRD_ANY", "Number of burst read transactions.",
BUS_TRAN_RFO, "BUS_TRAN_RFO", "Number of read for ownership transactions.",
BUS_TRANS_WB, "BUS_TRANS_WB", "Number of write back transactions.",
BUS_TRAN_IFETCH, "BUS_TRAN_IFETCH", "Number of instruction fetch transactions.",
BUS_TRAN_INVAL, "BUS_TRAN_INVAL", "Number of invalidate transactions.",
BUS_TRAN_PWR, "BUS_TRAN_PWR", "Number of partial write transactions.",
BUS_TRANS_P, "BUS_TRANS_P", "Number of partial transactions.",
BUS_TRANS_IO, "BUS_TRANS_IO", "Number of I/O transactions.",
BUS_TRAN_DEF, "BUS_TRAN_DEF", "Number of deferred transactions.",
BUS_TRAN_BURST, "BUS_TRAN_BURST", "Number of burst transactions.",
BUS_TRAN_ANY, "BUS_TRAN_ANY", "Number of all transactions.",
BUS_TRAN_MEM, "BUS_TRAN_MEM", "Number of memory transactions.",
BUS_DATA_RCV, "BUS_DATA_RCV", "Number of bus clock cycles during which this processor is receiving data.",
BUS_BNR_DRV, "BUS_BNR_DRV", "Number of bus clock cycles during which this processor is driving the BNR pin.",
BUS_HIT_DRV, "BUS_HIT_DRV", "Number of bus clock cycles during which this processor is driving the HIT pin.",
BUS_HITM_DRV, "BUS_HITM_DRV", "Number of bus clock cycles during which this processor is driving the HITM pin.",
BUS_SNOOP_STALL, "BUS_SNOOP_STALL", "Number of clock cycles during which the bus is snoop stalled.",
FLOPS, "FLOPS", "Number of computational floating-point operations retired.",
FP_COMP_OPS_EXE, "FP_COMP_OPS_EXE", "Number of computational floating-point operations executed.",
FP_ASSIST, "FP_ASSIST", "Number of floating- point exception cases handled by microcode.",
MUL, "MUL", "Number of multiplies.",
DIV, "DIV", "Number of divides.",
CYCLES_DIV_BUSY, "CYCLES_DIV_BUSY", " Number of cycles during which the divider is busy.",
LD_BLOCKS, "LD_BLOCKS", "Number of store buffer blocks.",
SB_DRAINS, "SB_DRAINS", "Number of store buffer drain cycles.",
MISALIGN_MEM_REF, "MISALIGN_MEM_REF", "Number of misaligned data memory references.",
INST_RETIRED, "INST_RETIRED", "Number of instructions retired.",
UOPS_RETIRED, "UOPS_RETIRED", "Number of UOPs retired.",
INST_DECODER, "INST_DECODER", "Number of instructions decoded.",
HW_INT_RX, "HW_INT_RX", "Number of hardware interrupts received.",
CYCLES_INT_MASKED, "CYCLES_INT_MASKED", "Number of processor cycles for which interrupts are disabled.",
CYCLES_INT_PENDING_AND_MASKED, "CYCLES_INT_PENDING_AND_MASKED", "Number of processor cycles for which interrupts are disabled and interrupts are pending.",
BR_INST_RETIRED, "BR_INST_RETIRED", "Number of branch instructions retired.",
BR_MISS_PRED_RETIRED, "BR_MISS_PRED_RETIRED", "Number of mispredicted branches retired.",
BR_TAKEN_RETIRED, "BR_TAKEN_RETIRED", "Number of taken branches retired.",
BR_MISS_PRED_NRET, "BR_MISS_PRED_NRET", "Number of taken mispredictions branches retired.",
BR_INST_DECODED, "BR_INST_DECODED", "Number of branch instructions decoded.",
BTB_MISSES, "BTB_MISSES", "Number of branches that miss the BTB.",
BR_BOGUS, "BR_BOGUS", "Number of bogus branches.",
BACLEARS, "BACLEARS", "Number of time BACLEAR is asserted.",
RESOURCE_STALLS, "RESOURCE_STALLS", "Number of cycles during which there are resource related stalls.",
PARTIAL_RAT_STALLS, "PARTIAL_RAT_STALLS", "Number of cycles or events for partial stalls.",
SEGMENT_REG_LOADS, "SEGMENT_REG_LOADS", "Number of segment register loads.",
CPU_CLK_UNHALTED, "CPU_CLK_UNHALTED", "Number of cycles during which the processor is not halted.",
-1, 0, 0
};

void usage(char *argv0) {
  int i = 0;
  printf("Usage: %s event1 event2 program args...\n", argv0);
  printf("event1 and event2 are one of the following events:\n");
  while (theEvents[i].shortDescription != 0) {
    printf("%s %s\n", theEvents[i].shortDescription, theEvents[i].longDescription);
    i++;
  }
}

int main(int argc, char **argv) {
  int fd; /* file description for ioctl */
  int i;  /* reusable iteration variable */
  int result; 
  char *event1, *event2;
  int eventId1, eventId2;

  if (argc < 4) {
    usage(argv[0]);
    exit(-1);
  }

  event1 = argv[1];
  event2 = argv[2];

  /* Map eventString to eventId */
  i = 0;
  while ((theEvents[i].shortDescription != 0) &&
	 strcmp(theEvents[i].shortDescription, event1) != 0) {
    i++;
  }
  
  eventId1 = theEvents[i].type;
  
  i = 0;
  while ((theEvents[i].shortDescription != 0) &&
	 strcmp(theEvents[i].shortDescription, event2) != 0) {
    i++;
  }
  eventId2 = theEvents[i].type;
  
  if (eventId1 < 0) {
    printf("Event %s is not a legal event\n", event1);
    exit(-1);
  }

  if (eventId2 < 0) {
    printf("Event %s is not a legal event\n", event2);
    exit(-1);
  }

  fd  = open("/dev/pmc", O_RDONLY, 0666);
  
  if (fd < 0) {
	perror("can't open /dev/pmc - is pmc driver installed?");
        exit(-1);
  } 

  result = ioctl(fd, 0, eventId1 | PMC_USER_MASK | PMC_OS_MASK | PMC_ENABLE_MASK);
  result = ioctl(fd, 1, eventId2 | PMC_USER_MASK | PMC_OS_MASK | PMC_ENABLE_MASK);

  close(fd);

  argc -= 3;
  argv = &argv[3];

  switch(fork()) {
  case -1:
    perror("Can't fork:");
    exit(-1);
  case 0:
    /* in the child */
    execvp(argv[0], argv);
    perror("can't exec");
    exit(-1);
    /* NOTREACHED */
    break;
  default:
    /* in the parent */
    {
      int status;
      wait(&status);
      break;
    }
  }
  {
    /* pull these out before printing, to minimize impact of printing */
    long long t1 = rdpmc(0);
    long long t2 = rdpmc(1);
    printf("%s: %Lu\n", event1, t1);
    printf("%s: %Lu\n", event2, t2);
  }
  exit(0);
}


